/**
  ****************************************************************************************************
  * @attention    
  * Copyright 2024 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS"AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
  *
  ****************************************************************************************************
  */
  
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "fm33ht0xxa_fl.h"
#include "iwdt.h"
#include "svd.h"
#include "rmu.h"
#include "led.h"
#include "uart.h"
#include "lin.h" 
#include "canfd.h"
#include "pll.h"
#include "bootloader.h"
#include "lptim16.h"

/**
  * @brief  HardFault 中断服务函数 请保留 
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{   /* 软件复位MCU，使用内联函数 */
    RMU_Soft_SystemReset();
}

static int32_t(*const TxMsgFill[BOOT_COMM_CHL])(uint8_t *, uint32_t) = 
{
    LIN_TxMsgFill, /* BOOT向LIN填充发送数据，LIN定义为通道0 */
    0,
    CANFD_TxMsgFill  /* BOOT向CANFD填充发送数据，CANFD定义为通道2 */
};

void FOUT0Init(uint32_t select)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    /* FOUT0--PC1 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_1;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    (void)FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
    
    FL_GPIO_SetFOUT0(GPIO, select);
}

int main(void)
{   
    /* 立即设置向量表 */
    SCB->VTOR = APP_START_ADDR;
    
    /* 使能IWDT */
    (void)IWDT_Init(FL_IWDT_PERIOD_4000MS);
    
    /* 延时函数初始化 */
    FL_Init();
    
    /* 使能SVD,阈值4.157V(falling)~4.257V(rising) */
    (void)SVD_Init(SVD_MONTIOR_VDD,FL_SVD_WARNING_THRESHOLD_GROUP11,FL_SVD_REFERENCE_1P0V,FL_SVD_Mode_LOWVOLTAGE_WARNING);
    
    /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
    while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 2000U/*us*/));
    
    /* 使能SVD低压复位功能,阈值2.657V(falling)~2.757V(rising) */
    (void)SVD_Init(SVD_MONTIOR_VDD,FL_SVD_WARNING_THRESHOLD_GROUP4,FL_SVD_REFERENCE_1P0V, FL_SVD_Mode_UNDERVOLTAGE_RESET);
    
    /* 检查APP执行标志是否置起，如置起则直接跳转到APP */
    BOOT_Object.AppExeFlagCheck();
    
    /* 无APP，执行BOOT参数配置 */
    BOOT_Object.Configuration();
    
    /* BOOT屏蔽总中断，APP此处需开启 */
    __set_PRIMASK(0);
    
    /* 先将外部XTHF分频到1M，再利用PLL锁相环倍频至32M，具体请见初始化参数 */
    (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
    
  #if (SYSCLK_FAULT_HANDING_MODE == SYSCLK_SWITCH_TO_BACKUP_CLOCK)
    /* 用于监控XTHF停振事件和PLL失锁事件，先配置系统时钟，再使能该监控定时器, 10ms周期 */
    LPTIM16_Init();
  #endif
  
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();  /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */
    
//    FOUT0Init(FL_GPIO_FOUT0_SELECT_PLL_DIV64);
        
    /* LIN初始化 */
    lin_init();
    
    /* CANFD外设初始化 */
    (void)CanFD_Init(CANFD);
    
    LED_ON(RED);   /* 调试信息灯 */
    FL_DelayMs(50);
    LED_OFF(RED);  
    while(1)
    {
        static uint32_t Time_10ms_Cnt = 0;
        
        /* 清狗 */
        IWDT_Clr();
        
        /* LIN接收帧处理 */
        if (0x1 == scheduleTable[0].FullFlg)
        {
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            scheduleTable[0].FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(scheduleTable[0].data, scheduleTable[0].length, 0/*LIN定义为通道0*/);
        }
        
        /* CANFD接收帧处理 */
        if (0x1 == CANFD_RxMessage.FullFlg)
        {         
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            CANFD_RxMessage.FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(CANFD_RxMessage.Data.Data8, (CANFD_RxMessage.PTR >> 28), 2/*CANFD定义为通道2*/);
        }        
        
        /* 功能执行 */
        BOOT_Object.Process(TxMsgFill);
        
      #if (SYSCLK_FAULT_HANDING_MODE == SYSCLK_SWITCH_TO_BACKUP_CLOCK)
        if (1 == Period10msFlag)
        {
            Period10msFlag = 0;
            /* 判断是否发生XTHF停振或PLL失锁事件，如有及时切回主系统时钟 */
            SwtichToMainSystemClock(&CMU_RCHF_Init_Data, &CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);

            /* 运行指示灯 */
            if (++Time_10ms_Cnt > 10)
            {
                LED_TOG(BLUE);
                Time_10ms_Cnt = 0;
            }
        }
      #endif
    }
}


